﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using System.Collections;

using SampleDataGeneratorNS;
using Infragistics.Olap.Data;
using Infragistics.Controls.Grids.Primitives;
using System.Collections.Specialized;

namespace SL_xamPivotGrid
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            Loaded += new RoutedEventHandler(DrillDown_Loaded);

        }

        void DrillDown_Loaded(object sender, RoutedEventArgs e)
        {

            IEnumerable sampleData = SampleDataGenerator.GenerateSales(500);

            //(xamPivotGrid1.DataSource as FlatDataSource).ItemsSource = sampleData;
            
            FlatDataSource flatDataSource = new FlatDataSource()
            {
                //Data is the IEnumerable to be used as a source
                ItemsSource = sampleData,
                Cube = DataSourceBase.GenerateInitialCube("Sale"),
                Rows = DataSourceBase.GenerateInitialItems("[Date]"),
                Columns = DataSourceBase.GenerateInitialItems("[City]"),
                Measures = DataSourceBase.GenerateInitialItems("AmountOfSale")
            };


            CubeMetadata cubeMetadata = new CubeMetadata { DataTypeFullName = "SampleDataGeneratorNS.Sale", DisplayName = "MY CUBE" };
            
            flatDataSource.CubesSettings.Add(cubeMetadata);

            DimensionMetadata numberOfUnitsMetadata = new DimensionMetadata
            {
                SourcePropertyName = "AmountOfSale",
                DisplayName = "Amount Of Sale (Avg)",
                DimensionType = DimensionType.Measure,
                AggregatorType = AggregatorType.Average,
                DisplayFormat = "{0:0.##}"
            };

            cubeMetadata.DimensionSettings.Add(numberOfUnitsMetadata);

            
            HierarchyDescriptor<Sale> sellerHierarchy = new HierarchyDescriptor<Sale>(p => p.Seller);
            sellerHierarchy.AddLevel(p => "All sellers", "All sellers");
            sellerHierarchy.AddLevel(p => p.Seller.Name, "Seller name");
            flatDataSource.AddHierarchyDescriptor(sellerHierarchy);
            HierarchyDescriptor<Sale> productHierarchy = new HierarchyDescriptor<Sale>(p => p.Product);
            productHierarchy.AddLevel(p => "All products", "All products");
            productHierarchy.AddLevel(p => p.Product.Name, "Product name");
            flatDataSource.AddHierarchyDescriptor(productHierarchy);
            /*
            HierarchyDescriptor<Sale> locationHierarchy = new HierarchyDescriptor<Sale>(p => p.City);
            locationHierarchy.AddLevel(p => "All locations", "All locations");
            locationHierarchy.AddLevel(p => p.City, "City");
            flatDataSource.AddHierarchyDescriptor(locationHierarchy);

            CubeMetadata cubeMetadata = new CubeMetadata();
            cubeMetadata.DisplayName = "Sales Data";
            cubeMetadata.DataTypeFullName = typeof(Sale).FullName;
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "AmountOfSale",
                DisplayName = "Amount of sale",
                DisplayFormat = "{0:C2}"
            });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
            {
                SourcePropertyName = "NumberOfUnits",
                DisplayName = "Units sold"
            });

            flatDataSource.CubesSettings.Add(cubeMetadata);
            */

            xamPivotGrid1.DataSource = flatDataSource;

            xamPivotDataSelector1.DataSource = flatDataSource;

            xamPivotGrid1.DataSource.Columns.CollectionChanged += new NotifyCollectionChangedEventHandler(Columns_CollectionChanged);

        }

        void DataSource_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
        }

        static LinearGradientBrush defaultFilterColor;

        void Columns_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                FilterViewModelBase obj = e.NewItems[0] as FilterViewModelBase;

                if (obj != null)
                {
                    foreach (IFilterMember filterMember in obj.FilterMembers)
                    {
                        filterMember.PropertyChanged += (s, args) =>
                        {
                            if (args.PropertyName == "IsSelected")
                            {
                                // Find the index of the column on which the event occurred...
                                int index = -1;
                                IFilterMember filter = s as IFilterMember;

                                foreach (Object maybeFilter in this.xamPivotGrid1.DataSource.Columns)
                                {
                                    index++;

                                    if (maybeFilter is IFilterViewModel)
                                    {
                                        IFilterViewModel fvm = (IFilterViewModel)maybeFilter;

                                        if (filter.FilterViewModel == fvm)
                                        {
                                            break;
                                        }
                                    }
                                }

                                // Determine if any of the filter members for the column are active
                                bool foundActiveFilter = false;

                                if (filter.FilterViewModel.FilterMembers != null)
                                {
                                    foreach (IFilterMember member in filter.FilterViewModel.FilterMembers)
                                    {
                                        // If any checkbox is "unchecked" (not selected), 
                                        // this is an "active" filter
                                        if (!(bool)member.IsSelected)
                                        {
                                            foundActiveFilter = true;
                                        }
                                    }
                                }

                                // If we found an active filter
                                if (foundActiveFilter)
                                {
                                    // Turn the column orange
                                    FieldItemControl item = (FieldItemControl)this.xamPivotGrid1.ColumnsDropControl.ItemContainerGenerator.ContainerFromIndex(index);

                                    if (defaultFilterColor == null)
                                        defaultFilterColor = (LinearGradientBrush)item.Background;

                                    item.Background = new SolidColorBrush(Colors.Orange);

                                }
                                else
                                {
                                    // Turn the column back to what it was
                                    FieldItemControl item = (FieldItemControl)this.xamPivotGrid1.ColumnsDropControl.ItemContainerGenerator.ContainerFromIndex(index);
                                    item.Background = defaultFilterColor;
                                }
                            }
                        };

                    }
                }
            }
        }

        public class Sale
        {
            public Product Product { get; set; }
            public Seller Seller { get; set; }
            public DateTime Date { get; set; }

            public string City
            {
                get { return Seller.City; }
                set { Seller.City = value; }
            }

            public int NumberOfUnits { get; set; }

            public double AmountOfSale
            {
                get { return Product.Price * NumberOfUnits; }
                set { Product.Price = value / NumberOfUnits; }
            }

            public string ProductName
            {
                get { return this.Product.Name; }
            }

            public string SallerName
            {
                get { return this.Seller.Name; }
            }

        }

        public class Product
        {
            public string Name { get; set; }
            public double Price { get; set; }
        }

        public class Seller
        {
            public string Name { get; set; }
            public string City { get; set; }
        }
    }
}
